#include "ve.h"
#include <stdio.h>

static cedrus_t *cedrus;

int ve_open()
{
	cedrus = cedrus_open();
	if (cedrus)
		return 1;
	return 0;
}

void ve_close()
{
	cedrus_close(cedrus);
}

int ve_get_version()
{
	return cedrus_get_ve_version(cedrus);
}

int ve_wait(int timeout)
{
	return cedrus_ve_wait(cedrus, timeout);
}

void *ve_get(int engine, uint32_t flags)
{
	return cedrus_ve_get(cedrus, engine, flags);
}

void ve_put(void)
{
	cedrus_ve_put(cedrus);
}


typedef struct mem_node {
	cedrus_mem_t *cedrus_mem;
	void *ptr;
} mem_node;

static mem_node mnodes[10] = {
{NULL, NULL},
{NULL, NULL},
{NULL, NULL},
{NULL, NULL},
{NULL, NULL},
{NULL, NULL},
{NULL, NULL},
{NULL, NULL},
{NULL, NULL},
{NULL, NULL},
};

static mem_node *find_empty_node()
{
	int i;
	for (i = 0; i < 10; i++) {
		if (mnodes[i].cedrus_mem == NULL) {
			fprintf(stderr, "%s() %p\n", __func__, &mnodes[i]);
			return &mnodes[i];
		}
	}

	fprintf(stderr, "%s() NO EMPTY NODE\n", __func__);
	return NULL;
}
static mem_node *find_node(void *ptr)
{
	int i;
	for (i = 0; i < 10; i++) {
		if (mnodes[i].ptr == ptr) {
		fprintf(stderr, "%s(%p) %p\n", __func__, ptr, &mnodes[i]);
			return &mnodes[i];
		}
	}

	fprintf(stderr, "%s() NO NODE for %p\n", __func__, ptr);
	return NULL;
}

void *ve_malloc(int size)
{
	mem_node *mnode = find_empty_node();
	mnode->cedrus_mem = cedrus_mem_alloc(cedrus, size);
	mnode->ptr = cedrus_mem_get_pointer(mnode->cedrus_mem);
	return mnode->ptr;
}

void ve_free(void *ptr)
{
	mem_node *mnode = find_node(ptr);
	cedrus_mem_free(mnode->cedrus_mem);
	mnode->cedrus_mem = NULL;
	mnode->ptr = NULL;
}

uint32_t ve_virt2phys(void *ptr)
{
	mem_node *mnode = find_node(ptr);
	return cedrus_mem_get_bus_addr(mnode->cedrus_mem);
}

void ve_flush_cache(void *start, int len)
{
	mem_node *mnode = find_node(start);
	cedrus_mem_flush_cache(mnode->cedrus_mem);
}

